@@ -1,58 +1,59 @@
 /* ====================================================================
- * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
+ * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
+ *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
  *
- * 4. The names "Apache Server" and "Apache Group" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For written permission, please contact
- *    apache@apache.org.
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
  *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
  *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
- *
- * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
- * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
  * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  * ====================================================================
  *
  * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Group and was originally based
- * on public domain software written at the National Center for
- * Supercomputing Applications, University of Illinois, Urbana-Champaign.
- * For more information on the Apache Group and the Apache HTTP server
- * project, please see <http://www.apache.org/>.
+ * individuals on behalf of the Apache Software Foundation.  For more
+ * information on the Apache Software Foundation, please see
+ * <http://www.apache.org/>.
  *
+ * Portions of this software are based upon public domain software
+ * originally written at the National Center for Supercomputing Applications,
+ * University of Illinois, Urbana-Champaign.
  */
 
 #define CORE_PRIVATE
@@ -89,11 +90,6 @@
  * group are the first three arguments to be passed; if not, all three
  * must be NULL.  The query info is split into separate arguments, where
  * "+" is the separator between keyword arguments.
- *
- * XXXX: note that the WIN32 code uses one of the suexec strings 
- * to pass an interpreter name.  Remember this if changing the way they
- * are handled in create_argv.
- *
  */
 static char **create_argv(pool *p, char *path, char *user, char *group,
 			  char *av0, const char *args)
@@ -267,16 +263,16 @@ API_EXPORT(void) ap_add_common_vars(request_rec *r)
 #endif
 
 #ifdef OS2
-    if (env_temp = getenv("COMSPEC")) {
+    if ((env_temp = getenv("COMSPEC")) != NULL) {
         ap_table_addn(e, "COMSPEC", env_temp);            
     }
-    if (env_temp = getenv("ETC")) {
+    if ((env_temp = getenv("ETC")) != NULL) {
         ap_table_addn(e, "ETC", env_temp);            
     }
-    if (env_temp = getenv("DPATH")) {
+    if ((env_temp = getenv("DPATH")) != NULL) {
         ap_table_addn(e, "DPATH", env_temp);            
     }
-    if (env_temp = getenv("PERLLIB_PREFIX")) {
+    if ((env_temp = getenv("PERLLIB_PREFIX")) != NULL) {
         ap_table_addn(e, "PERLLIB_PREFIX", env_temp);            
     }
 #endif
@@ -284,7 +280,8 @@ API_EXPORT(void) ap_add_common_vars(request_rec *r)
     ap_table_addn(e, "PATH", env_path);
     ap_table_addn(e, "SERVER_SIGNATURE", ap_psignature("", r));
     ap_table_addn(e, "SERVER_SOFTWARE", ap_get_server_version());
-    ap_table_addn(e, "SERVER_NAME", ap_get_server_name(r));
+    ap_table_addn(e, "SERVER_NAME", 
+		  ap_escape_html(r->pool,ap_get_server_name(r)));
     ap_table_addn(e, "SERVER_ADDR", r->connection->local_ip);	/* Apache */
     ap_table_addn(e, "SERVER_PORT",
 		  ap_psprintf(r->pool, "%u", ap_get_server_port(r)));
@@ -649,6 +646,63 @@ API_EXPORT(int) ap_scan_script_header_err_buff(request_rec *r, BUFF *fb,
     return ap_scan_script_header_err_core(r, buffer, getsfunc_BUFF, fb);
 }
 
+struct vastrs {
+    va_list args;
+    int arg;
+    const char *curpos;
+};
+
+static int getsfunc_STRING(char *w, int len, void *pvastrs)
+{
+    struct vastrs *strs = (struct vastrs*) pvastrs;
+    char *p;
+    int t;
+    
+    if (!strs->curpos || !*strs->curpos) 
+        return 0;
+    p = strchr(strs->curpos, '\n');
+    if (p)
+        ++p;
+    else
+        p = strchr(strs->curpos, '\0');
+    t = p - strs->curpos;
+    if (t > len)
+        t = len;
+    strncpy (w, strs->curpos, t);
+    w[t] = '\0';
+    if (!strs->curpos[t]) {
+        ++strs->arg;
+        strs->curpos = va_arg(strs->args, const char *);
+    }
+    else
+        strs->curpos += t;
+    return t;    
+}
+
+/* ap_scan_script_header_err_strs() accepts additional const char* args...
+ * each is treated as one or more header lines, and the first non-header
+ * character is returned to **arg, **data.  (The first optional arg is
+ * counted as 0.)
+ */
+API_EXPORT_NONSTD(int) ap_scan_script_header_err_strs(request_rec *r, 
+                                                      char *buffer, 
+                                                      const char **termch,
+                                                      int *termarg, ...)
+{
+    struct vastrs strs;
+    int res;
+
+    va_start(strs.args, termarg);
+    strs.arg = 0;
+    strs.curpos = va_arg(strs.args, char*);
+    res = ap_scan_script_header_err_core(r, buffer, getsfunc_STRING, (void *) &strs);
+    if (termch)
+        *termch = strs.curpos;
+    if (termarg)
+        *termarg = strs.arg;
+    va_end(strs.args);
+    return res;
+}
 
 API_EXPORT(void) ap_send_size(size_t size, request_rec *r)
 {
@@ -663,63 +717,24 @@ API_EXPORT(void) ap_send_size(size_t size, request_rec *r)
 	ap_rputs("   1k", r);
     }
     else if (size < 1048576) {
-	ap_rprintf(r, "%4dk", (size + 512) / 1024);
+	ap_rprintf(r, "%4dk", (int)((size + 512) / 1024));
     }
     else if (size < 103809024) {
 	ap_rprintf(r, "%4.1fM", size / 1048576.0);
     }
     else {
-	ap_rprintf(r, "%4dM", (size + 524288) / 1048576);
+	ap_rprintf(r, "%4dM", (int)((size + 524288) / 1048576));
     }
 }
 
-#if defined(OS2) || defined(WIN32)
-static char **create_argv_cmd(pool *p, char *av0, const char *args, char *path)
-{
-    register int x, n;
-    char **av;
-    char *w;
-
-    for (x = 0, n = 2; args[x]; x++) {
-        if (args[x] == '+') {
-	    ++n;
-	}
-    }
-
-    /* Add extra strings to array. */
-    n = n + 2;
-
-    av = (char **) ap_palloc(p, (n + 1) * sizeof(char *));
-    av[0] = av0;
-
-    /* Now insert the extra strings we made room for above. */
-    av[1] = strdup("/C");
-    av[2] = strdup(path);
-
-    for (x = (1 + 2); x < n; x++) {
-	w = ap_getword(p, &args, '+');
-	ap_unescape_url(w);
-	av[x] = ap_escape_shell_cmd(p, w);
-    }
-    av[n] = NULL;
-    return av;
-}
-#endif
-
-
 API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
 			     char **env, int shellcmd)
 {
     int pid = 0;
-#if defined(RLIMIT_CPU)  || defined(RLIMIT_NPROC) || \
-    defined(RLIMIT_DATA) || defined(RLIMIT_VMEM) || defined (RLIMIT_AS)
-
     core_dir_config *conf;
     conf = (core_dir_config *) ap_get_module_config(r->per_dir_config,
 						    &core_module);
 
-#endif
-
 #if !defined(WIN32) && !defined(OS2)
     /* the fd on r->server->error_log is closed, but we need somewhere to
      * put the error messages from the log_* functions. So, we use stderr,
@@ -786,8 +801,11 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
         int env_len, e;
         char *env_block, *env_block_pos;
 
-	if (r->args && r->args[0] && !strchr(r->args, '='))
+	if ((conf->cgi_command_args != AP_FLAG_OFF)
+            && r->args && r->args[0]
+            && !strchr(r->args, '=')) {
 	    args = r->args;
+        }
 	    
 	program = fopen(r->filename, "rt");
 	
@@ -899,11 +917,11 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
     {
         /* Adapted from Alec Kloss' work for OS/2 */
         char *interpreter = NULL;
+        char *invokename = NULL;
         char *arguments = NULL;
         char *ext = NULL;
-        char *exename = NULL;
         char *s = NULL;
-        char *quoted_filename;
+        char *t = NULL;
         char *pCommand;
         char *pEnvBlock, *pNext;
 
@@ -932,11 +950,42 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
                 return (pid);
             }
 
+            if (interpreter && *interpreter 
+                    && (s = strstr(interpreter, "\"%1\""))) {
+                s[1] = '\0';
+                s += 3;
+                invokename = ap_pstrdup(r->pool, r->filename);
+            }
+            else
+            {
+                char shortname[MAX_PATH];
+                DWORD rv = GetShortPathName(r->filename, shortname, MAX_PATH);
+                if (!rv || rv >= MAX_PATH) {
+                    ap_log_rerror(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, r,
+                                  "%s is not executable; cannot translate "
+                                  "to a short path name.", r->filename);
+                    return (pid);
+                }
+                invokename = ap_pstrdup(r->pool, shortname);
+
+                if (interpreter && *interpreter
+                        && (s = strstr(interpreter, "%1"))) {
+                    s[0] = '\0';
+                    s += 2;
+                }
+            }
+            for (t = invokename; *t; ++t) {
+                if (*t == '/')
+                    *t = '\\';
+            }
+
             /*
              * Look at the arguments...
              */
             arguments = "";
-            if ((r->args) && (r->args[0]) && !strchr(r->args, '=')) { 
+            if ((conf->cgi_command_args != AP_FLAG_OFF)
+                 && (r->args) && (r->args[0])
+                 && !strchr(r->args, '=')) { 
                 /* If we are in this leg, there are some other arguments
                  * that we must include in the execution of the CGI.
                  * Because CreateProcess is the way it is, we have to
@@ -968,41 +1017,92 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
        
                 /*
                  * We need to unescape any characters that are 
-                 * in the arguments list.
+                 * in the arguments list.  Truncate to 4000
+                 * characters for safety, being careful of the
+                 * now-escaped characters.
                  */
                 ap_unescape_url(arguments);
                 arguments = ap_escape_shell_cmd(r->pool, arguments);
+                if (strlen(arguments) > 4000)
+                {
+                    int len = 4000;
+                    while (len && arguments[len - 1] == '\\') {
+                        --len;
+                    }
+                    arguments[len] = '\0';
+                }
+
+                /*
+                 * Now that the arguments list is 'shell' escaped with
+                 * backslashes, we need to make cmd.exe/command.com 
+                 * safe from this same set of characters.
+                 */
+                if (fileType == eCommandShell32) {
+                    arguments = ap_caret_escape_args(r->pool, arguments);
+                }
+                else if (fileType == eCommandShell16) {
+                    arguments = ap_pstrcat(r->pool, "\"", 
+                            ap_double_quotes(r->pool, arguments), "\"", NULL);
+                }
             }
 
             /*
-             * We have the interpreter (if there is one) and we have 
-             * the arguments (if there are any).
-             * Build the command string to pass to CreateProcess. 
+             * The remaining code merges the interpreter, the backslashed
+             * and potentially shortened invoke name, the various
+             * interpreter segments and the arguments.
+             *
+             * Note that interpreter started out with %1 %* arguments,
+             * so the *t character skips the %* arguments list, and the
+             * *s already skipped the %1 argument (quoted or not.)
              */
-            quoted_filename = ap_pstrcat(r->pool, "\"", r->filename, "\"", NULL);
-            if (interpreter && *interpreter) {
-                pCommand = ap_pstrcat(r->pool, interpreter, " ", 
-                                      quoted_filename, " ", arguments, NULL);
+
+            if (s && (t = strstr(s, "%*"))) {
+                /* interpreter formatted: prog [opts] %1 [opts] %* [opts] 
+                 */
+                t[0] = '\0';
+                t += 2;
+                pCommand = ap_pstrcat(r->pool, interpreter, invokename,
+                                               s, arguments, t, NULL);
+            }
+            else if (s) {
+                /* interpreter formatted: prog [opts] %1 [opts] 
+                 */
+                pCommand = ap_pstrcat(r->pool, interpreter, invokename,
+                                               s, " ", arguments, NULL);
+            }
+            else if (interpreter) {
+                /* interpreter formatted: prog [opts]
+                 */
+                pCommand = ap_pstrcat(r->pool, interpreter, " ", invokename,
+                                               " ", arguments, NULL);
             }
             else {
-                pCommand = ap_pstrcat(r->pool, quoted_filename, " ", arguments, NULL);
+                /* no interpreter required
+                 */
+                pCommand = ap_pstrcat(r->pool, invokename, 
+                                               " ", arguments, NULL);
             }
 
-         } else {
-
-            char *shell_cmd = "CMD.EXE /C ";
-            OSVERSIONINFO osver;
-            osver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-         
-            /*
-             * Use CMD.EXE for NT, COMMAND.COM for WIN95
-             */
-            if (GetVersionEx(&osver)) {
-                if (osver.dwPlatformId != VER_PLATFORM_WIN32_NT) {
-                    shell_cmd = "COMMAND.COM /C ";
-                }
-            }       
-            pCommand = ap_pstrcat(r->pool, shell_cmd, argv0, NULL);
+        }
+        else /* shellcmd */
+        {
+            char *p, *comspec = getenv("COMSPEC");
+            const char *quotecomspec;
+            const char *quoteargv0;
+            if (!comspec)
+                comspec = SHELL_PATH;
+            p = strchr(comspec, '\0');
+            quotecomspec = (strchr(comspec, ' ') && comspec[0] != '\"')
+                         ? "\"" : "";
+            quoteargv0 = (strchr(argv0, ' ') && argv0[0] != '\"') ? "\"" : "";
+            pCommand = ap_pstrcat(r->pool, quotecomspec, comspec, quotecomspec,
+                                  " /c ", quoteargv0, argv0, quoteargv0, NULL);
+            /* Forward slash argv[0] only */
+            for (p = pCommand + strlen(pCommand) - strlen(argv0) 
+                              - strlen(quoteargv0); *p; ++p) {
+                if (*p == '/')
+                    *p = '\\';
+            }
         }
 
         /*
@@ -1010,7 +1110,7 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
          * and make sure it does not show on screen.
          */
         si.cb = sizeof(si);
-        si.dwFlags     = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
+        si.dwFlags     = STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW;
         si.wShowWindow = SW_HIDE;
         si.hStdInput   = pinfo->hPipeInputRead;
         si.hStdOutput  = pinfo->hPipeOutputWrite;
@@ -1038,12 +1138,19 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
             i++;
         }
 
+        ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_INFO, r->server,
+                     "Invoking CGI Command '%s'", pCommand);
+        for (i = 0; env[i]; ++i) {
+            ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_DEBUG, r->server,
+                         "  CGI env[%d] = '%s'", i, env[i]);
+        }
+
         if (CreateProcess(NULL, pCommand, NULL, NULL, TRUE, 
                           0,
                           pEnvBlock,
                           ap_make_dirstr_parent(r->pool, r->filename),
                           &si, &pi)) {
-            if (fileType == eFileTypeEXE16) {
+            if (fileType == eFileTypeEXE16 || fileType == eCommandShell16) {
                 /* Hack to get 16-bit CGI's working. It works for all the 
                  * standard modules shipped with Apache. pi.dwProcessId is 0 
                  * for 16-bit CGIs and all the Unix specific code that calls 
@@ -1127,7 +1234,9 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
 		   NULL, env);
 	}
 
-	else if ((!r->args) || (!r->args[0]) || strchr(r->args, '=')) {
+	else if ((conf->cgi_command_args == AP_FLAG_OFF)
+            || (!r->args) || (!r->args[0])
+            || strchr(r->args, '=')) {
 	    execle(SUEXEC_BIN, SUEXEC_BIN, execuser, grpname, argv0,
 		   NULL, env);
 	}
@@ -1144,7 +1253,9 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
 	    execle(SHELL_PATH, SHELL_PATH, "-c", argv0, NULL, env);
 	}
 
-	else if ((!r->args) || (!r->args[0]) || strchr(r->args, '=')) {
+	else if ((conf->cgi_command_args == AP_FLAG_OFF)
+            || (!r->args) || (!r->args[0])
+            || strchr(r->args, '=')) {
 	    execle(r->filename, argv0, NULL, env);
 	}
 
