@@ -76,8 +76,8 @@ int ap_proxy_http_canon(request_rec *r, char *url, const char *scheme, int def_p
     const char *err;
     int port;
 
-    /* do syntatic check.
-     * We break the URL into host, port, path, search
+    /*
+     * do syntatic check. We break the URL into host, port, path, search
      */
     port = def_port;
     err = ap_proxy_canon_netloc(r->pool, &url, NULL, NULL, &host, &port);
@@ -85,9 +85,10 @@ int ap_proxy_http_canon(request_rec *r, char *url, const char *scheme, int def_p
         return HTTP_BAD_REQUEST;
 
     /* now parse path/search args, according to rfc1738 */
-    /* N.B. if this isn't a true proxy request, then the URL _path_
-     * has already been decoded.  True proxy requests have r->uri
-     * == r->unparsed_uri, and no others have that property.
+    /*
+     * N.B. if this isn't a true proxy request, then the URL _path_ has
+     * already been decoded.  True proxy requests have r->uri ==
+     * r->unparsed_uri, and no others have that property.
      */
     if (r->uri == r->unparsed_uri) {
         search = strchr(url, '?');
@@ -146,12 +147,12 @@ static const char *proxy_location_reverse_map(request_rec *r, const char *url)
  * route.)
  */
 int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
-                       const char *proxyhost, int proxyport)
+                              const char *proxyhost, int proxyport)
 {
     const char *strp;
     char *strp2;
     const char *err, *desthost;
-    int i, j, sock, len, backasswards;
+    int i, j, sock,/* len,*/ backasswards;
     table *req_hdrs, *resp_hdrs;
     array_header *reqhdrs_arr;
     table_entry *reqhdrs_elts;
@@ -163,18 +164,22 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
     char portstr[32];
     pool *p = r->pool;
     int destport = 0;
+    int chunked = 0;
     char *destportstr = NULL;
     const char *urlptr = NULL;
     const char *datestr, *urlstr;
+    int result, major, minor;
+    const char *content_length;
 
     void *sconf = r->server->module_config;
     proxy_server_conf *conf =
-    (proxy_server_conf *) ap_get_module_config(sconf, &proxy_module);
+    (proxy_server_conf *)ap_get_module_config(sconf, &proxy_module);
     struct noproxy_entry *npent = (struct noproxy_entry *) conf->noproxies->elts;
     struct nocache_entry *ncent = (struct nocache_entry *) conf->nocaches->elts;
     int nocache = 0;
 
-    if (conf->cache.root == NULL) nocache = 1;
+    if (conf->cache.root == NULL)
+        nocache = 1;
 
     memset(&server, '\0', sizeof(server));
     server.sin_family = AF_INET;
@@ -213,7 +218,7 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
     for (i = 0; i < conf->noproxies->nelts; i++) {
         if (destaddr.s_addr == npent[i].addr.s_addr ||
             (npent[i].name != NULL &&
-              (npent[i].name[0] == '*' || strstr(desthost, npent[i].name) != NULL)))
+             (npent[i].name[0] == '*' || strstr(desthost, npent[i].name) != NULL)))
             return ap_proxyerror(r, HTTP_FORBIDDEN,
                                  "Connect to remote machine blocked");
     }
@@ -232,30 +237,31 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
     }
 
 
-    /* we have worked out who exactly we are going to connect to, now
-     * make that connection...
+    /*
+     * we have worked out who exactly we are going to connect to, now make
+     * that connection...
      */
     sock = ap_psocket(p, PF_INET, SOCK_STREAM, IPPROTO_TCP);
     if (sock == -1) {
         ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
-                    "proxy: error creating socket");
+                      "proxy: error creating socket");
         return HTTP_INTERNAL_SERVER_ERROR;
     }
 
 #if !defined(TPF) && !defined(BEOS)
     if (conf->recv_buffer_size) {
         if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF,
-                       (const char *) &conf->recv_buffer_size, sizeof(int))
+                       (const char *)&conf->recv_buffer_size, sizeof(int))
             == -1) {
             ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
-                         "setsockopt(SO_RCVBUF): Failed to set ProxyReceiveBufferSize, using default");
+                          "setsockopt(SO_RCVBUF): Failed to set ProxyReceiveBufferSize, using default");
         }
     }
 #endif
 
 #ifdef SINIX_D_RESOLVER_BUG
     {
-        struct in_addr *ip_addr = (struct in_addr *) *server_hp.h_addr_list;
+        struct in_addr *ip_addr = (struct in_addr *)*server_hp.h_addr_list;
 
         for (; ip_addr->s_addr != 0; ++ip_addr) {
             memcpy(&server.sin_addr, ip_addr, sizeof(struct in_addr));
@@ -280,30 +286,31 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
             return DECLINED;    /* try again another way */
         else
             return ap_proxyerror(r, HTTP_BAD_GATEWAY, ap_pstrcat(r->pool,
-                                "Could not connect to remote machine: ",
-                                strerror(errno), NULL));
+                                    "Could not connect to remote machine: ",
+                                                    strerror(errno), NULL));
     }
 
     /* record request_time for HTTP/1.1 age calculation */
     c->req_time = time(NULL);
 
-    /* build upstream-request headers by stripping r->headers_in from
-     * connection specific headers.
-     * We must not remove the Connection: header from r->headers_in,
-     * we still have to react to Connection: close
+    /*
+     * build upstream-request headers by stripping r->headers_in from
+     * connection specific headers. We must not remove the Connection: header
+     * from r->headers_in, we still have to react to Connection: close
      */
     req_hdrs = ap_copy_table(r->pool, r->headers_in);
     ap_proxy_clear_connection(r->pool, req_hdrs);
 
-    /* At this point, we start sending the HTTP/1.1 request to the
-     * remote server (proxy or otherwise).
+    /*
+     * At this point, we start sending the HTTP/1.1 request to the remote
+     * server (proxy or otherwise).
      */
     f = ap_bcreate(p, B_RDWR | B_SOCKET);
     ap_bpushfd(f, sock, sock);
 
     ap_hard_timeout("proxy send", r);
     ap_bvputs(f, r->method, " ", proxyhost ? url : urlptr, " HTTP/1.1" CRLF,
-           NULL);
+              NULL);
     /* Send Host: now, adding it to req_hdrs wouldn't be much better */
     if (destportstr != NULL && destport != DEFAULT_HTTP_PORT)
         ap_bvputs(f, "Host: ", desthost, ":", destportstr, CRLF, NULL);
@@ -313,43 +320,68 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
     if (conf->viaopt == via_block) {
         /* Block all outgoing Via: headers */
         ap_table_unset(req_hdrs, "Via");
-    } else if (conf->viaopt != via_off) {
+    }
+    else if (conf->viaopt != via_off) {
         /* Create a "Via:" request header entry and merge it */
         i = ap_get_server_port(r);
-        if (ap_is_default_port(i,r)) {
-            strcpy(portstr,"");
-        } else {
+        if (ap_is_default_port(i, r)) {
+            strcpy(portstr, "");
+        }
+        else {
             ap_snprintf(portstr, sizeof portstr, ":%d", i);
         }
         /* Generate outgoing Via: header with/without server comment: */
         ap_table_mergen(req_hdrs, "Via",
-                    (conf->viaopt == via_full)
+                        (conf->viaopt == via_full)
                         ? ap_psprintf(p, "%d.%d %s%s (%s)",
-                                HTTP_VERSION_MAJOR(r->proto_num),
-                                HTTP_VERSION_MINOR(r->proto_num),
-                                ap_get_server_name(r), portstr,
-                                SERVER_BASEVERSION)
+                                      HTTP_VERSION_MAJOR(r->proto_num),
+                                      HTTP_VERSION_MINOR(r->proto_num),
+                                      ap_get_server_name(r), portstr,
+                                      SERVER_BASEVERSION)
                         : ap_psprintf(p, "%d.%d %s%s",
-                                HTTP_VERSION_MAJOR(r->proto_num),
-                                HTTP_VERSION_MINOR(r->proto_num),
-                                ap_get_server_name(r), portstr)
-                        );
+                                      HTTP_VERSION_MAJOR(r->proto_num),
+                                      HTTP_VERSION_MINOR(r->proto_num),
+                                      ap_get_server_name(r), portstr)
+            );
     }
 
-    /* Add X-Forwarded-For: so that the upstream has a chance to
-       determine, where the original request came from. */
-    ap_table_mergen(req_hdrs, "X-Forwarded-For", r->connection->remote_ip);
-    
+    /* the X-* headers are only added if we are a reverse
+     * proxy, otherwise we would be giving away private information.
+     */
+    if (r->proxyreq == PROXY_PASS) {
+        const char *buf;
+
+        /*
+         * Add X-Forwarded-For: so that the upstream has a chance to determine,
+         * where the original request came from.
+         */
+        ap_table_mergen(req_hdrs, "X-Forwarded-For", r->connection->remote_ip);
+
+        /* Add X-Forwarded-Host: so that upstream knows what the
+         * original request hostname was.
+         */
+        if ((buf = ap_table_get(r->headers_in, "Host"))) {
+            ap_table_mergen(req_hdrs, "X-Forwarded-Host", buf);
+        }
+
+        /* Add X-Forwarded-Server: so that upstream knows what the
+         * name of this proxy server is (if there are more than one)
+         * XXX: This duplicates Via: - do we strictly need it?
+         */
+        ap_table_mergen(req_hdrs, "X-Forwarded-Server", r->server->server_hostname);
+    } 
+
     /* we don't yet support keepalives - but we will soon, I promise! */
     ap_table_set(req_hdrs, "Connection", "close");
 
     reqhdrs_arr = ap_table_elts(req_hdrs);
-    reqhdrs_elts = (table_entry *) reqhdrs_arr->elts;
+    reqhdrs_elts = (table_entry *)reqhdrs_arr->elts;
     for (i = 0; i < reqhdrs_arr->nelts; i++) {
         if (reqhdrs_elts[i].key == NULL || reqhdrs_elts[i].val == NULL
 
-        /* Clear out hop-by-hop request headers not to send:
-         * RFC2616 13.5.1 says we should strip these headers:
+        /*
+         * Clear out hop-by-hop request headers not to send: RFC2616 13.5.1
+         * says we should strip these headers:
          */
             || !strcasecmp(reqhdrs_elts[i].key, "Host") /* Already sent */
             || !strcasecmp(reqhdrs_elts[i].key, "Keep-Alive")
@@ -357,16 +389,16 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
             || !strcasecmp(reqhdrs_elts[i].key, "Trailer")
             || !strcasecmp(reqhdrs_elts[i].key, "Transfer-Encoding")
             || !strcasecmp(reqhdrs_elts[i].key, "Upgrade")
-
-            /* XXX: @@@ FIXME: "Proxy-Authorization" should *only* be 
-             * suppressed if THIS server requested the authentication,
-             * not when a frontend proxy requested it!
-             *
-             * The solution to this problem is probably to strip out
-             * the Proxy-Authorisation header in the authorisation
-             * code itself, not here. This saves us having to signal
-             * somehow whether this request was authenticated or not.
-             */
+        /*
+         * XXX: @@@ FIXME: "Proxy-Authorization" should *only* be suppressed
+         * if THIS server requested the authentication, not when a frontend
+         * proxy requested it!
+         * 
+         * The solution to this problem is probably to strip out the
+         * Proxy-Authorisation header in the authorisation code itself, not
+         * here. This saves us having to signal somehow whether this request
+         * was authenticated or not.
+         */
             || !strcasecmp(reqhdrs_elts[i].key, "Proxy-Authorization"))
             continue;
         ap_bvputs(f, reqhdrs_elts[i].key, ": ", reqhdrs_elts[i].val, CRLF, NULL);
@@ -375,105 +407,122 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
     /* the obligatory empty line to mark the end of the headers */
     ap_bputs(CRLF, f);
 
-    /* send the request data, if any. */
-    if (ap_should_client_block(r)) {
-        while ((i = ap_get_client_block(r, buffer, sizeof buffer)) > 0)
-            ap_bwrite(f, buffer, i);
-    }
+    /* and flush the above away */
     ap_bflush(f);
+
+    /* and kill the send timeout */
     ap_kill_timeout(r);
 
 
-    /* Right - now it's time to listen for a response.
+    /* read the request data, and pass it to the backend.
+     * we might encounter a stray 100-continue reponse from a PUT or POST,
+     * if this happens we ignore the 100 continue status line and read the
+     * response again.
      */
-    ap_hard_timeout("proxy receive", r);
+    {
+        /* send the request data, if any. */
+        ap_hard_timeout("proxy receive request data", r);
+        if (ap_should_client_block(r)) {
+            while ((i = ap_get_client_block(r, buffer, sizeof buffer)) > 0) {
+                ap_reset_timeout(r);
+                ap_bwrite(f, buffer, i);
+            }
+        }
+        ap_bflush(f);
+        ap_kill_timeout(r);
 
-    len = ap_bgets(buffer, sizeof buffer - 1, f);
-    if (len == -1) {
-        ap_bclose(f);
+
+        /* then, read a response line */
+        ap_hard_timeout("proxy receive response status line", r);
+        result = ap_proxy_read_response_line(f, r, buffer, sizeof(buffer)-1, &backasswards, &major, &minor);
         ap_kill_timeout(r);
-        ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
-                     "ap_bgets() - proxy receive - Error reading from remote server %s (length %d)",
-                     proxyhost ? proxyhost : desthost, len);
-        return ap_proxyerror(r, HTTP_BAD_GATEWAY,
-                             "Error reading from remote server");
-    } else if (len == 0) {
-        ap_bclose(f);
-        ap_kill_timeout(r);
-        return ap_proxyerror(r, HTTP_BAD_GATEWAY,
-                             "Document contains no data");
+
+        /* trap any errors */
+        if (result != OK) {
+            ap_bclose(f);
+            return result;
+        }
+
+        /* if this response was 100-continue, a stray response has been caught.
+         * read the line again for the real response
+         */
+        if (r->status == 100) {
+            ap_hard_timeout("proxy receive response status line", r);
+            result = ap_proxy_read_response_line(f, r, buffer, sizeof(buffer)-1, &backasswards, &major, &minor);
+            ap_kill_timeout(r);
+
+            /* trap any errors */
+            if (result != OK) {
+                ap_bclose(f);
+                return result;
+            }
+        }
     }
 
-    /* Is it an HTTP/1 response?
-     * Do some sanity checks on the response.
-     * (This is buggy if we ever see an HTTP/1.10)
+
+    /*
+     * We have our response status line from the convoluted code above,
+     * now we read the headers to continue.
      */
-    if (ap_checkmask(buffer, "HTTP/#.# ###*")) {
-        int major, minor;
-        if (2 != sscanf(buffer, "HTTP/%u.%u", &major, &minor)) {
-            /* if no response, default to HTTP/1.1 - is this correct? */
-            major = 1;
-            minor = 1;
-        }
+    ap_hard_timeout("proxy receive response headers", r);
 
-        /* If not an HTTP/1 message or if the status line was > 8192 bytes */
-        if (buffer[5] != '1' || buffer[len - 1] != '\n') {
-            ap_bclose(f);
-            ap_kill_timeout(r);
-            return HTTP_BAD_GATEWAY;
-        }
-        backasswards = 0;
-        buffer[--len] = '\0';
-
-        buffer[12] = '\0';
-        r->status = atoi(&buffer[9]);
-        buffer[12] = ' ';
-        r->status_line = ap_pstrdup(p, &buffer[9]);
+    /*
+     * Is it an HTTP/1 response? Do some sanity checks on the response. (This
+     * is buggy if we ever see an HTTP/1.10)
+     */
+    if (backasswards == 0) {
 
         /* read the response headers. */
         /* N.B. for HTTP/1.0 clients, we have to fold line-wrapped headers */
         /* Also, take care with headers with multiple occurences. */
 
-        resp_hdrs = ap_proxy_read_headers(r, buffer, HUGE_STRING_LEN, f);
+        resp_hdrs = ap_proxy_read_headers(r, buffer, sizeof(buffer), f);
         if (resp_hdrs == NULL) {
-            ap_log_error(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, r->server,
-                 "proxy: Bad HTTP/%d.%d header returned by %s (%s)",
-                 major, minor, r->uri, r->method);
+            ap_log_error(APLOG_MARK, APLOG_WARNING | APLOG_NOERRNO, r->server,
+                         "proxy: Bad HTTP/%d.%d header returned by %s (%s)",
+                         major, minor, r->uri, r->method);
             resp_hdrs = ap_make_table(p, 20);
-            nocache = 1;    /* do not cache this broken file */
+            nocache = 1;        /* do not cache this broken file */
         }
 
         /* handle Via header in the response */
         if (conf->viaopt != via_off && conf->viaopt != via_block) {
             /* Create a "Via:" response header entry and merge it */
             i = ap_get_server_port(r);
-            if (ap_is_default_port(i,r)) {
-                strcpy(portstr,"");
-            } else {
+            if (ap_is_default_port(i, r)) {
+                strcpy(portstr, "");
+            }
+            else {
                 ap_snprintf(portstr, sizeof portstr, ":%d", i);
             }
             ap_table_mergen((table *)resp_hdrs, "Via",
                             (conf->viaopt == via_full)
                             ? ap_psprintf(p, "%d.%d %s%s (%s)",
-                                major, minor,
-                                ap_get_server_name(r), portstr,
-                                SERVER_BASEVERSION)
+                                          major, minor,
+                                          ap_get_server_name(r), portstr,
+                                          SERVER_BASEVERSION)
                             : ap_psprintf(p, "%d.%d %s%s",
-                                major, minor,
-                                ap_get_server_name(r), portstr)
-                            );
+                                          major, minor,
+                                          ap_get_server_name(r), portstr)
+                );
         }
 
-        /* strip hop-by-hop headers defined by Connection */
+        /* is this content chunked? */
+        chunked = ap_find_last_token(r->pool,
+                                     ap_table_get(resp_hdrs, "Transfer-Encoding"),
+                                     "chunked");
+
+        /* strip hop-by-hop headers defined by Connection and RFC2616 */
         ap_proxy_clear_connection(p, resp_hdrs);
-        /* Now add out bound headers set by other modules */
-        resp_hdrs = ap_overlay_tables(r->pool, r->err_headers_out, resp_hdrs);
-}
+
+        content_length = ap_table_get(resp_hdrs, "Content-Length");
+        if (content_length != NULL) {
+            c->len = ap_strtol(content_length, NULL, 10);
+        }
+
+    }
     else {
         /* an http/0.9 response */
-        backasswards = 1;
-        r->status = 200;
-        r->status_line = "200 OK";
 
         /* no headers */
         resp_hdrs = ap_make_table(p, 20);
@@ -482,10 +531,11 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
     ap_kill_timeout(r);
 
     /*
-     * HTTP/1.1 requires us to accept 3 types of dates, but only generate
-     * one type
+     * HTTP/1.1 requires us to accept 3 types of dates, but only generate one
+     * type
      */
-    /* we SET the dates here, obliterating possible multiple dates, as only
+    /*
+     * we SET the dates here, obliterating possible multiple dates, as only
      * one of each date makes sense in each response.
      */
     if ((datestr = ap_table_get(resp_hdrs, "Date")) != NULL)
@@ -497,72 +547,82 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
 
     /* handle the ProxyPassReverse mappings */
     if ((urlstr = ap_table_get(resp_hdrs, "Location")) != NULL)
-      ap_table_set(resp_hdrs, "Location", proxy_location_reverse_map(r, urlstr));
+        ap_table_set(resp_hdrs, "Location", proxy_location_reverse_map(r, urlstr));
     if ((urlstr = ap_table_get(resp_hdrs, "URI")) != NULL)
-      ap_table_set(resp_hdrs, "URI", proxy_location_reverse_map(r, urlstr));
+        ap_table_set(resp_hdrs, "URI", proxy_location_reverse_map(r, urlstr));
     if ((urlstr = ap_table_get(resp_hdrs, "Content-Location")) != NULL)
-      ap_table_set(resp_hdrs, "Content-Location", proxy_location_reverse_map(r , urlstr));
+        ap_table_set(resp_hdrs, "Content-Location", proxy_location_reverse_map(r, urlstr));
 
 /* check if NoCache directive on this host */
     if (nocache == 0) {
         for (i = 0; i < conf->nocaches->nelts; i++) {
             if (destaddr.s_addr == ncent[i].addr.s_addr ||
                 (ncent[i].name != NULL &&
-                  (ncent[i].name[0] == '*' ||
-                   strstr(desthost, ncent[i].name) != NULL))) {
-               nocache = 1;
-               break;
+                 (ncent[i].name[0] == '*' ||
+                  strstr(desthost, ncent[i].name) != NULL))) {
+                nocache = 1;
+                break;
             }
         }
 
-    /* update the cache file, possibly even fulfilling the request if
-     * it turns out a conditional allowed us to serve the object from the
-     * cache...
-     */
-    i = ap_proxy_cache_update(c, resp_hdrs, !backasswards, nocache);
-    if (i != DECLINED) {
-        ap_bclose(f);
-        return i;
-    }
+        /*
+         * update the cache file, possibly even fulfilling the request if it
+         * turns out a conditional allowed us to serve the object from the
+         * cache...
+         */
+        i = ap_proxy_cache_update(c, resp_hdrs, !backasswards, nocache);
+        if (i != DECLINED) {
+            ap_bclose(f);
+            return i;
+        }
 
         /* write status line and headers to the cache file */
         ap_proxy_write_headers(c, ap_pstrcat(p, "HTTP/1.1 ", r->status_line, NULL), resp_hdrs);
     }
 
     /* Setup the headers for our client from upstreams response-headers */
-    ap_overlap_tables(r->headers_out, resp_hdrs, AP_OVERLAP_TABLES_SET);
+    ap_proxy_table_replace(r->headers_out, resp_hdrs);
     /* Add X-Cache header - be careful not to obliterate any upstream headers */
     ap_table_mergen(r->headers_out, "X-Cache",
-                  ap_pstrcat(r->pool, "MISS from ",
-                             ap_get_server_name(r), NULL));
+                    ap_pstrcat(r->pool, "MISS from ",
+                               ap_get_server_name(r), NULL));
     /* The Content-Type of this response is the upstream one. */
-    r->content_type = ap_table_get (r->headers_out, "Content-Type");
-    ap_log_error(APLOG_MARK, APLOG_DEBUG|APLOG_NOERRNO, r->server, "Content-Type: %s", r->content_type);
+    r->content_type = ap_table_get(r->headers_out, "Content-Type");
+    ap_log_error(APLOG_MARK, APLOG_DEBUG | APLOG_NOERRNO, r->server, "Content-Type: %s", r->content_type);
 
     /* finally output the headers to the client */
     ap_send_http_header(r);
 
-    /* Is it an HTTP/0.9 respose? If so, send the extra data we read
-       from upstream as the start of the reponse to client */
+    /*
+     * Is it an HTTP/0.9 respose? If so, send the extra data we read from
+     * upstream as the start of the reponse to client
+     */
+/* FIXME: This code is broken: we try and write a buffer and length that
+ * were never intelligently initialised. Rather have a bit of broken protocol
+ * handling for now than broken code.
+ */
+/*
     if (backasswards) {
         ap_hard_timeout("proxy send assbackward", r);
 
         ap_bwrite(r->connection->client, buffer, len);
         if (c != NULL && c->fp != NULL && ap_bwrite(c->fp, buffer, len) != len) {
             ap_log_rerror(APLOG_MARK, APLOG_ERR, c->req,
-                "proxy: error writing extra data to %s", c->tempfile);
+                      "proxy: error writing extra data to %s", c->tempfile);
             c = ap_proxy_cache_error(c);
         }
         ap_kill_timeout(r);
     }
-
+*/
 
 #ifdef CHARSET_EBCDIC
-    /* What we read/write after the header should not be modified
-     * (i.e., the cache copy is ASCII, not EBCDIC, even for text/html)
+    /*
+     * What we read/write after the header should not be modified (i.e., the
+     * cache copy is ASCII, not EBCDIC, even for text/html)
      */
-    ap_bsetflag(f, B_ASCII2EBCDIC|B_EBCDIC2ASCII, 0);
-    ap_bsetflag(r->connection->client, B_ASCII2EBCDIC|B_EBCDIC2ASCII, 0);
+    r->ebcdic.conv_in = r->ebcdic.conv_out = 0;
+    ap_bsetflag(f, B_ASCII2EBCDIC | B_EBCDIC2ASCII, 0);
+    ap_bsetflag(r->connection->client, B_ASCII2EBCDIC | B_EBCDIC2ASCII, 0);
 #endif
 
 /* send body */
@@ -578,7 +638,7 @@ int ap_proxy_http_handler(request_rec *r, cache_req *c, char *url,
  * content length is not known. We need to make 100% sure c->len is always
  * set correctly before we get here to correctly do keepalive.
  */
-        ap_proxy_send_fb(f, r, c, c->len, 0, conf->io_buffer_size);
+        ap_proxy_send_fb(f, r, c, c->len, 0, chunked, conf->io_buffer_size);
     }
 
     /* ap_proxy_send_fb() closes the socket f for us */
